#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "macro.h"
#include "opcodes.h"
#include "util.h"



/* Hash of MACRO_SIZE macrolists */
struct macrolist *mactabs[MACRO_SIZE];


void macInit(void)
{
  int i;

  for (i=0; i<MACRO_SIZE; i++) {
    mactabs[i] = NULL;
  }
}


void macDestroy(void)
{
  int i;
  struct macrolist *mp;

  for (i=0; i<MACRO_SIZE; i++) {
    while (mactabs[i] != NULL) {
      mp = (mactabs[i])->next;
      free((mactabs[i])->label);
      stDestroy(mactabs[i]->s);
      free(mactabs[i]);
      mactabs[i] = mp;
    }
  }
}


StateList ** macAdd(const char *buf, unsigned int f)
{
  int i;
  struct macrolist *ment;

  if (!buf[0]) {
    fprintf(stderr, "WARNING: Attempt to add null macro name. Ignoring...\n");
    return NULL;
  }

  // fprintf(stderr, "%04x symAdd: Adding symbol '%s' = %04x, flags = %04x\n",
  //     lc, buf, loc, f);
  // fprintf(stderr, "     Pearson hash = %02x\n", Pearson_hash((void *)buf, strlen(buf)));
  i = (buf[0] % MACRO_SIZE);
  ment = (struct macrolist *)malloc(sizeof(struct macrolist));
  ment->label = strdup(buf);
  ment->s = stInit();
  ment->invocations = 0;
  ment->flags = f;
  ment->next = mactabs[i];
  mactabs[i] = ment;
  return ment->s;
}


StateList * macLookup(const char *lbl)
{
  int i;
  struct macrolist *sp;

  i = (lbl[0] % MACRO_SIZE);
  sp = mactabs[i];
  while (sp) {
    if (!strcmp(sp->label, lbl)) return *(sp->s);
    sp = sp->next;
  }
  return NULL;
}


int macInvocations(const char *lbl, const int increment)
{
  int i;
  struct macrolist *sp;

  i = (lbl[0] % MACRO_SIZE);
  sp = mactabs[i];
  while (sp) {
    if (!strcmp(sp->label, lbl)) {
      (sp->invocations) += increment;
      return sp->invocations;
    }
    sp = sp->next;
  }
  return -1;
}


// Reset macro state (i.e. invocation counts) for next pass
//
void macReset(void)
{
  int i;
  struct macrolist *mp;

  for (i=0; i<MACRO_SIZE; i++) {
    mp = mactabs[i];
    while (mp) {
      mp->invocations = 0;
      mp = mp->next;
    }  // while
  }  // for
}


void macDump(FILE *lfp)
{
  int i;
  struct macrolist *sp;

  for (i=0; i<MACRO_SIZE; i++) {
    sp = mactabs[i];
    while (sp != NULL) {
      fprintf(lfp, "\tmacro %s (%d %04x)\n", sp->label, sp->invocations, sp->flags);
      stDump(sp->s, lfp);
      fprintf(lfp, "\tendm %s\n", sp->label);
      sp = sp->next;
    }
  }
}


