#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "statements.h"
#include "opcodes.h"
#include "util.h"




StateList ** stInit(void)
{
  StateList **sl;

  sl = (StateList **)malloc(sizeof(StateList *));
  *sl = NULL;
  return sl;
}


void stDestroy(StateList **sl)
{
  StateList *sp;

  if (sl == NULL) return;
  while ((*sl) != NULL) {
    sp = (*sl)->next;
    free((*sl)->label);
    free((*sl)->opcode);
    free((*sl)->operands);
    free((*sl)->comment);
    free(*sl);
    *sl = sp;
  }
  free(sl);
  sl = NULL;
}


void stAdd(StateList **sl, const char *l, const char *op,
	const char *operands, const char *c)
{
  StateList *sent;
  StateList *sp;

  if (sl == NULL) {
    fprintf(stderr, "ERROR: Statement List pointer is null (was stInit called?)\n");
    exit(1);
  }

  // Append each statement to the end. Order matters here.
  //
  // fprintf(stderr, "Allocating a StateList element, opcode %s\n", op);
  sent = (StateList *)malloc(sizeof(StateList));
  sent->next = NULL;
  sent->label = strdup(l);
  sent->opcode = strdup(op);
  sent->operands = strdup(operands);
  sent->comment = strdup(c);

  if (*sl == NULL) {
    // fprintf(stderr, "Storing first entry\n");
    *sl = sent;
  } else {
    // fprintf(stderr, "Seeking to end of list\n");
    sp = *sl;
    while (sp->next != NULL) sp = sp->next;
    // fprintf(stderr, "Setting sp->next\n");
    sp->next = sent;
  }
  // fprintf(stderr, "Leaving stAdd()\n");
}


void stUse(StateList **sl, char *l, char *op, char *operands, char *comment)
{
  strcpy(l, (*sl)->label);
  strcpy(op, (*sl)->opcode);
  strcpy(operands, (*sl)->operands);
  strcpy(comment,  (*sl)->comment);
  *sl = (*sl)->next;
}


void stDump(StateList **sl, FILE *lfp)
{
  StateList *sp;

  if (lfp == NULL) return;
  sp = *sl;
  while (sp != NULL) {
    fprintf(lfp, "%s\t%s %s\t%s\n", sp->label, sp->opcode, sp->operands,
        sp->comment);
    sp = sp->next;
  }
}


